<?php
/**
 * Layout Engine themes repository
 *
 * @package WordPress
 * @subpackage layout_engine
 * @since 1.0.0.1
 */


class LE_Theme_Repo
{
	/**
	 * Create schedule and hook to run the theme updater after certain interval
	 *
	 * @access public
	 * @since 1.0.0.1
	 */	
	public static function init()
	{
		add_action('init', array('LE_Theme_Repo','wp_init' ));
		
		add_action('admin_init',array('LE_Theme_Repo', 'admin_init'));
		add_action('admin_init', array('LE_Theme_Repo','maybe_update_themes' ));		
	}
	
	/**
	 * Create schedule hook
	 *
	 * @access public
	 * @since 1.0.0.1
	 */	
	public static function wp_init()
	{
		if ( !wp_next_scheduled('wp_update_layout_engine_themes'))
			wp_schedule_event(time(), 'twicedaily', 'wp_update_layout_engine_themes');	
	
		add_action('wp_update_layout_engine_themes', array('LE_Theme_Repo','wp_update_themes' ));
	}
	
	/**
	 * Provide Layout Engine Themes repository support
	 *
	 * @access public
	 * @since 1.0.0.1
	 */	
	public static function admin_init()
	{
		add_filter( 'themes_api', array('LE_Theme_Repo','themes_api' ), 10, 3);
		add_action('install_themes_pre_dashboard', array('LE_Theme_Repo','install_themes_pre_dashboard' ));
	}
	
	/**
	 * Query Layout Engine Repository if Layout Engine feature is selected
	 *
 	 * @param string $content theme list
 	 * @param string $action
     * @param array|object $args Optional. Arguments to serialize for the Theme Info API.
	 * @access public
	 * @since 1.0.0.1
	 */	
	public static function themes_api($content, $action, $args = null)
	{
		if(!empty($args) && is_object($args))
		{
			$is_custom_repo = false;
			if(isset($args->tag) && is_array($args->tag) && in_array('layout-engine', $args->tag))
			{
				$is_custom_repo = true;
				$layout_engine = array_search('layout-engine', $args->tag);
				unset($args->tag[$layout_engine]);
					
				if(empty($args->tag)) unset($args->tag);				
			}elseif((isset($args->slug)) && (strpos($args->slug, 'layout-engine-') !== false))
			{
				$args->slug = substr($args->slug,strlen('layout-engine-'));
				$is_custom_repo = true;
			}
			
			if($is_custom_repo)
			{
				$request = wp_remote_post('http://simpleux.co.uk/plugins/wordpress/layout-engine/repository/theme', array( 'body' => array('action' => $action, 'request' => serialize($args))) );
				
				if ( is_wp_error($request) ) {
					$content = new WP_Error('themes_api_failed', __( 'An unexpected error occurred. Something may be wrong with simpleux.co.uk or this server&#8217;s configuration. If you continue to have problems, please try the <a href="http://wordpress.org/support/plugin/layout-engine/">support forums</a>.' ), $request->get_error_message() );
				} else {
					$content = maybe_unserialize( wp_remote_retrieve_body( $request ) );
					
					if ( ! is_object( $content ) && ! is_array( $content ) )
						$content = new WP_Error('themes_api_failed', __( 'An unexpected error occurred. Something may be wrong with simpleux.co.uk or this server&#8217;s configuration. If you continue to have problems, please try the <a href="http://wordpress.org/support/plugin/layout-engine/">support forums</a>.' ), wp_remote_retrieve_body( $request ) );
				}

				$content = apply_filters('themes_api_result', $content, $action, $args);
				
			}
		}
		
		return $content;
	}
	
	/**
	 * Replace Theme Install dashboard to support layout engine as feature
	 *
	 * @access public
	 * @since 1.0.0.1
	 */	
	public static function install_themes_pre_dashboard()
	{
		remove_action('install_themes_dashboard', 'install_themes_dashboard');
		add_action('install_themes_dashboard', array('LE_Theme_Repo','install_themes_dashboard'));
	}
	
	/**
	 * WordPress Theme install dashboard with Layout Engine as feature
	 *
	 * @access public
	 * @since 1.0.0.1
	 */
	public static function install_themes_dashboard()
	{
		install_theme_search_form( false );
		?>
		<h4><?php _e('Feature Filter') ?></h4>
		<p class="install-help"><?php _e( 'Find a theme based on specific features.' ); ?></p>
		
		<form method="get" action="">
			<input type="hidden" name="tab" value="search" />
			<?php
			$feature_list_default = (array) get_theme_feature_list( );
			$feature_list = array('Repositories' => array('layout-engine' => 'Layout Engine Themes'));
			$feature_list += $feature_list_default;
			
			echo '<div class="feature-filter">';
		
			foreach ( (array) $feature_list as $feature_name => $features ) {
				$feature_name = esc_html( $feature_name );
				echo '<div class="feature-name">' . $feature_name . '</div>';
		
				echo '<ol class="feature-group">';
				foreach ( $features as $feature => $feature_name ) {
					$feature_name = esc_html( $feature_name );
					$feature = esc_attr($feature);
		?>
		
		<li>
			<input type="checkbox" name="features[]" id="feature-id-<?php echo $feature; ?>" value="<?php echo $feature; ?>" />
			<label for="feature-id-<?php echo $feature; ?>"><?php echo $feature_name; ?></label>
		</li>
		
		<?php	} ?>
		</ol>
		<br class="clear" />
		<?php
			} ?>
		
		</div>
		<br class="clear" />
		<?php submit_button( __( 'Find Themes' ), 'button', 'search' ); ?>
		</form>
		<?php	
	}	

	/**
	 * Check themes versions only after a duration of time.
	 *
	 * @access public
	 * @since 1.0.0.1
	 */
	public static function maybe_update_themes( ) 
	{
		$current = get_site_transient( 'update_themes_layout_engine' );
		if ( isset( $current->last_checked ) && 43200 > ( time( ) - $current->last_checked ) )
			return;
	
		LE_Theme_Repo::wp_update_themes();
	}	
	
	/**
	 * Check theme versions against the latest versions hosted on simpleux.org.uk
	 *
	 * @access public
	 * @since 1.0.0.1
	 * 
	 * @return mixed Returns null if update is unsupported. Returns false if check is too soon.
	 */
	public static function wp_update_themes() {
		include ABSPATH . WPINC . '/version.php'; // include an unmodified $wp_version

		if ( defined( 'WP_INSTALLING' ) )
			return false;
	
		$installed_themes = wp_get_themes();
		$last_update = get_site_transient( 'update_themes_layout_engine' );
		if ( ! is_object($last_update) )
			$last_update = new stdClass;
	
		$themes = array();
		$checked = array();
	
		foreach ( $installed_themes as $theme ) 
		{
			$tags = (array) $theme->get('Tags');
			
			if(in_array('layout-engine', $tags))
			{
					$checked[ $theme->get_stylesheet() ] = $theme->get('Version');
				
					$themes[ $theme->get_stylesheet() ] = array
					(
							'Name'       => $theme->get('Name'),
							'Version'    => $theme->get('Version'),
							
					);
			}
		}
		
		// Check for update on a different schedule, depending on the page.
		switch ( current_filter() ) {
			case 'load-update-core.php' :
				$timeout = 60; // 1 min
				break;
			case 'load-themes.php' :
			case 'load-update.php' :
				$timeout = 3600; // 1 hour
				break;
			default :
				$timeout = 43200; // 12 hours
		}
	
		//$time_not_changed = isset( $last_update->last_checked ) && $timeout > ( time( ) - $last_update->last_checked );
		$time_not_changed = false;
	
		if ( $time_not_changed ) {
			$theme_changed = false;
			foreach ( $checked as $slug => $v ) {
				$update_request->checked[ $slug ] = $v;
	
				if ( !isset( $last_update->checked[ $slug ] ) || strval($last_update->checked[ $slug ]) !== strval($v) )
					$theme_changed = true;
			}
	
			if ( isset ( $last_update->response ) && is_array( $last_update->response ) ) {
				foreach ( $last_update->response as $slug => $update_details ) {
					if ( ! isset($checked[ $slug ]) ) {
						$theme_changed = true;
						break;
					}
				}
			}
	
			// Bail if we've checked recently and if nothing has changed
			if ( ! $theme_changed )
				return false;
		}
	
		// Update last_checked for current to prevent multiple blocking requests if request hangs
		$last_update->last_checked = time();
		set_site_transient( 'update_themes_layout_engine', $last_update );
	
		$options = array(
				'timeout' 			=> ( ( defined('DOING_CRON') && DOING_CRON ) ? 30 : 3),
				'body'				=> array(
													'action' => 'check_updates',
													'request' => serialize(array( 'themes' =>  $themes ) ),
									   ),
				'user-agent'		=> 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )
		);
		
		$raw_response = wp_remote_post( 'http://simpleux.co.uk/plugins/wordpress/layout-engine/repository/theme', $options );
	
		if ( is_wp_error( $raw_response ) || 200 != wp_remote_retrieve_response_code( $raw_response ) )
			return false;
	
		$new_update = new stdClass;
		$new_update->last_checked = time( );
		$new_update->checked = $checked;
		
		$response = maybe_unserialize( wp_remote_retrieve_body( $raw_response ) );
		
		//Update WordPress theme upgrade settings
		if ( is_array( $response ) )
		{
			$new_update->response = $response;
			
			$wordpress_themes_last_update = (object) get_site_transient( 'update_themes' );
			$wordpress_themes_last_update->last_checked = $new_update->last_checked;
			
			if(!$wordpress_themes_last_update->checked) $wordpress_themes_last_update->checked = array();
			
			foreach($new_update->checked as $theme_name => $theme_version)
				$wordpress_themes_last_update->checked[$theme_name] = $theme_version;
			
			if(!$wordpress_themes_last_update->response) $wordpress_themes_last_update->response = array();
			
			foreach($new_update->response as $theme_name => $theme_upgrade_info)
			{
				$wordpress_themes_last_update->response[$theme_name] = $theme_upgrade_info;
			}
			
			set_site_transient( 'update_themes', $wordpress_themes_last_update);
		}
		
		set_site_transient( 'update_themes_layout_engine', $new_update );
	}	
	
}

LE_Theme_Repo::init();


?>